"""
Copyright (c) 2008, Carl Scheffler
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * The name of the copyright owner may not be used to endorse or promote
      products derived from this software without specific prior written
      permission.

THIS SOFTWARE IS PROVIDED BY <copyright holder> ``AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

from __future__ import division
from render import render_epi, render_camm, render_rml
import Tkinter
import sys

class Application(Tkinter.Toplevel):
    def __init__(self, window = (960, 480), title = ''):
        # Top level
        self.root = Tkinter.Tk()
        self.root.title(title)
        
        self.canvasframe = Tkinter.Frame(self.root)
        self.xscrollbar = Tkinter.Scrollbar(self.canvasframe, orient=Tkinter.HORIZONTAL)
        self.xscrollbar.grid(row=1, column=0, sticky=Tkinter.E+Tkinter.W)
        self.yscrollbar = Tkinter.Scrollbar(self.canvasframe)
        self.yscrollbar.grid(row=0, column=1, sticky=Tkinter.N+Tkinter.S)
        
        self.canvas = Tkinter.Canvas(self.canvasframe, width=window[0], height=window[1],
                                     background='white', xscrollcommand=self.xscrollbar.set,
                                     yscrollcommand=self.yscrollbar.set)
        self.canvas.grid(row=0, column=0, sticky=Tkinter.N+Tkinter.S+Tkinter.E+Tkinter.W)
        self.canvas.configure(scrollregion=(0,0,window[0],window[1]))
        self.xscrollbar.config(command=self.canvas.xview)
        self.yscrollbar.config(command=self.canvas.yview)
        
        self.canvasframe.pack()

        self.loadframe = Tkinter.Frame(self.root)
        self.loadtext = Tkinter.Entry(self.loadframe)
        self.loadtext.grid(row=0, column=0)
        self.loadbutton = Tkinter.Button(self.loadframe, text='Load', command=self.load_click)
        self.loadbutton.grid(row=0, column=1)
        self.loadframe.pack()
        
    def mainloop(self, **kwargs):
        self.root.mainloop(**kwargs)

    def load_click(self):
        def is_epi(commands):
            magic_string = "%-12345X"
            return commands[:len(magic_string)] == magic_string

        def is_camm(commands):
            magic_string = "PA;PA;!ST1;"
            return commands[:len(magic_string)] == magic_string

        def is_rml(commands):
            magic_string = "PA;PA;VS"
            return commands[:len(magic_string)] == magic_string
        
        filename = self.loadtext.get()
        infile = file(filename, 'rb')
        commands = infile.read()
        infile.close()

        success = False
        file_formats = [(is_epi, render_epi), (is_camm, render_camm), (is_rml, render_rml)]
        for (check_function, render_function) in file_formats:
            if check_function(commands):
                render_function(commands, canvas=self.canvas)
                success = True

        if not success:
            print "File format not supported."


app = Application(title='FabLab machine simulator')
app.mainloop()
